/**
 *	Written 2010 by Tyler Montbriand.
 *	Freeware.
 */

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include "tcgets.h"
#include "sensor8.h"

enum
{
	MODE_REPLY=0x0001,
	MODE_BINARY=0x0002,
};

#define READ(FD, V)	read_n(FD, &V, sizeof(V))

//int read_n(int fd, void *buf, int n);
//int write_n(int fd, const void *buf, int n);
int sensor_get_text(int fd);
void interact(int fd, unsigned char cmd, int mode);

#if 0
int main(void)
{
	temp_t t;
	const char *dev="/dev/ttyUSB0";
	int fd=termios_open(dev, "115200N81");
	if(fd < 0)
	{
		perror(dev);
		return(1);
	}

//	interact(fd, CMD_SET_ASCII, 0);
	interact(fd, CMD_VIN(1), MODE_REPLY);

	fprintf(stderr, "Calibration data begins\n");

	fprintf(stderr, "2.5V\n");
	interact(fd, CMD_VIN(0), MODE_REPLY);
	interact(fd, CMD_VIN(0), MODE_REPLY|MODE_BINARY);
	interact(fd, CMD_SET1(1), 0);
	fprintf(stderr, "5.0v\n");
	interact(fd, CMD_VIN(0), MODE_REPLY);
	interact(fd, CMD_VIN(0), MODE_REPLY|MODE_BINARY);
	interact(fd, CMD_SET0(1), 0);
	fprintf(stderr, "0.0v\n");
	interact(fd, CMD_VIN(0), MODE_REPLY);
	interact(fd, CMD_VIN(0), MODE_REPLY|MODE_BINARY);

	fprintf(stderr, "Calibration data ends\n");
	interact(fd, CMD_VIN(1), MODE_REPLY);
	interact(fd, CMD_VIN(0), MODE_REPLY);
//	interact(fd, CMD_SET_BINARY, 0);
//	interact(fd, CMD_VIN(0), MODE_REPLY|MODE_BINARY);
//	fprintf(stderr, "temp is %.1f degrees C\n", get_temp(fd,0)/16.0f);


//	interact(fd, CMD_SET_BINARY, 0);
//	interact(fd, CMD_TEMP_IN(0), MODE_REPLY|MODE_BINARY);
//	interact(fd, CMD_SET_FAHRENHEIT, 0);
//	interact(fd, CMD_TEMP_IN(0), MODE_REPLY|MODE_BINARY);

//	interact(fd, CMD_SET_BINARY, 0);

//	write(fd, &c, 1);
//	write(fd, temp_in+0, 1);
//	READ(fd, temp);

//	fprintf(stderr, "temp is 0x%04x\n", (unsigned int)temp);
//	sensor_get_text(fd);
//	bytes=read(fd, buf, 128);
//	fprintf(stderr, "Read %d bytes: '\n", (int)bytes);
//	fwrite(buf, bytes, 1, stderr);
//	fprintf(stderr, "'\n");
	
	close(fd);
	return(0);
}
#endif

int read_n(int fd, void *buf, int n)
{
	int pos=0;

	while(pos < n)
	{
		ssize_t s=read(fd, ((char *)buf)+pos, n-pos);
		if(s <= 0)	continue;
		pos += s;
//		fprintf(stderr, "Got %d/%d\n", pos, n);
	}

	return(pos);
}

int sensor_get_text(int fd)
{
	char buf[64]; int pos=0;

	while(pos < 63)
	{
		ssize_t s=read(fd, buf+pos, 63-pos);
		if(s <= 0)	continue;
		pos += s;
//		fprintf(stderr, "Got %d bytes\n", (int)s);
		buf[pos]='\0';
		if(buf[pos-1] == '\r')
		{
			pos-=2;
			break;
		}
	}

	buf[pos++]='\0';
	fprintf(stderr, "Got '%s'\n", buf);
	return(0);
}

void interact(int fd, unsigned char cmd, int mode)
{
	char bin=CMD_SET_ASCII;
	if(mode & MODE_BINARY)
		bin=CMD_SET_BINARY;

	if(mode && MODE_REPLY)	write(fd, &bin, 1);

	if(write(fd, &cmd, 1) < 1)
		fprintf(stderr, "Couldn't write 0x%02x\n", cmd);
//	else	fprintf(stderr, "Wrote 0x%02x\n", cmd);

	if(mode & MODE_REPLY)
	{
		if(mode & MODE_BINARY)
		{
			temp_t s;
			READ(fd, s);
			fprintf(stderr, "got 0x%04x aka %.2fV aka %.2f C\n", 
				(unsigned short int)s,
				BIN16_TO_VOLT((unsigned short)s),
				BIN16_TO_TEMP(s));
		}
		else
		{
			sensor_get_text(fd);
		}
	}
}

unsigned short get_volt(int fd, int sensor)
{
	unsigned char cmdbuf[2]={CMD_SET_BINARY, 0x00};
	int pos=0;
	unsigned short temp;
	if(sensor > 7)	return(0xffff);

	cmdbuf[1]=CMD_VIN(sensor);

	write_n(fd, cmdbuf, 2);
	READ(fd, temp);
	return(temp);
}


temp_t get_temp(int fd, int sensor)
{
	int tries=0;
	unsigned char cmdbuf[2]={CMD_SET_BINARY, 0x00};
	int pos=0;
	short int temp;

	if(sensor > 7)	return(0xffff);

	cmdbuf[1]=CMD_TEMP_IN(sensor);

GET_TEMP_RETRY:
	write_n(fd, cmdbuf, 2);
	READ(fd, temp);

	if(temp == TEMP_ERROR)
	{
		if(tries)	// Only retry once
			return(TEMP_ERROR);
		else	tries=1;

		goto GET_TEMP_RETRY;
	}

	return(temp);
}

int write_n(int fd, const void *buf, int n)
{
	int pos=0;

	while(pos < n)
	{
		ssize_t s=write(fd, ((const unsigned char *)buf)+pos, n-pos);
		if(s <= 0) continue;
		pos += s;
	}

	return(pos);
}

const char temp_in[8]	=	"90-=OP[]";
const char digital_in[8]=	"ASDFGHJK";	// Pattern apparent now?
const char analog_in[8]	=	"ZXCVBNM,";
const char set_hi[8]	=	"12345678";
const char set_lo[8]	=	"QWERTYUI";
